require_relative 'lib/helpers'

FactoryBot.define do
  factory :_donation, class: SBF::Client::FullDonation do
    to_create { |it| raise "Failed to create #{it.class}: #{it.errors.details}" unless it.save }

    initialize_with do
      new(attributes)
    end

    status { SBF::Client::Donation::Status::ACTIVE }
    amount { rand(10..100) }
    display_name { Faker::Name.name }
    year { DateTime.now.year }
    fund { nil }
    memorial { nil }
  end

  factory :donation, class: SBF::Client::FullDonation, parent: :_donation do
    after :build do |donation, evaluator|
      donation.payment_details = build(:cc_payment) if unpopulated?(evaluator, :payment_details)
      donation.donor = build(:person) if unpopulated?(evaluator, :donor)
    end
  end

  factory :cc_payment, class: SBF::Client::Payment::CreditCardDetails do
  end

  factory :cash_donation, class: SBF::Client::FullDonation, parent: :_donation do
    status { SBF::Client::Donation::Status::PENDING }
    how_created { SBF::Client::Donation::HowCreated::SUBMIT_CASH }

    after :build do |donation, evaluator|
      donation.payment_details = build(:cash_payment) if unpopulated?(evaluator, :payment_details)
      donation.donor = build(:person, status: SBF::Client::Person::Status::PENDING) if unpopulated?(evaluator, :donor)
      donation.submitter = build(:person) if unpopulated?(evaluator, :submitter)
    end
  end

  factory :cash_payment, class: SBF::Client::Payment::CashDetails do
  end

  factory :nonce_payment, class: SBF::Client::Payment::NonceDetails do
    gateway { SBF::Client::Payment::Gateway::PAYMENTSPRING }
    nonce { fake_valid_nonce }
    device_data { random }
  end
end

require 'httparty'
require 'uri'
def fake_valid_nonce
  url = "#{ENV['PAYMENTSPRING_BASE_URL']}/tokens/jsonp"
  pub_key = ENV['PAYMENTSPRING_PUBLIC_KEY']
  data = {
    public_api_key: pub_key,
    card_number: '4111111111111111',
    card_exp_month: '1',
    card_exp_year: Date.today.year + 1,
    card_owner_name: 'John%20Doe'
  }

  uri = URI.parse("#{url}?#{data.to_query}")
  response = HTTParty.get(uri, headers: {'Accept' => 'application/json'})
  raise 'Error retrieving valid nonce' unless response.ok?

  response.parsed_response['id']
end
