require_relative 'lib/helpers'

FactoryBot.define do
  factory :event, class: SBF::Client::FullEvent do
    to_create { |it| raise "Failed to create #{it.class}: #{it.errors.details}" unless it.save }

    initialize_with do
      new(attributes)
    end

    event_title { "#{Faker::Team.name} #{random}" }
    year { Time.now.year }
    date { Time.now.to_s }
    fund_relationship_type { 'no_association' }
    status { 'active' }
    type { 'head_shaving' }
    sub_type { 'community' }
    how_created { 'admin_control_panel' }
    fundraising_goal { rand(20) * 5000 }
    is_private { false }

    after :build do |event, evaluator|
      event.venue = build(:event_venue) if unpopulated?(evaluator, :venue)
      event.contacts = build(:event_contacts) if unpopulated?(evaluator, :contacts)
      event.organizers = [build(:lead_organizer, person: build(:person))] if unpopulated?(evaluator, :organizers)
    end
  end

  factory :event_activity, class: SBF::Client::Event::Activity do
    skip_create

    initialize_with do
      new(attributes)
    end

    start_time { '12:00:00' }
    end_time { '14:00:00' }
    activity { random }
    sort_order { 1 }
  end
end
