require_relative 'lib/helpers'

FactoryBot.define do
  factory :kid, class: SBF::Client::FullKid do
    to_create do |it|
      it.relationships.first.person = FactoryBot.create(:person) if it.relationships.first.person.nil?
      raise "Failed to create #{it.class}: #{it.errors.details}" unless it.save
    end

    initialize_with do
      new(attributes)
    end

    birthday { Faker::Date.birthday(min_age: 5, max_age: 25).to_s }
    web_message { 'Thank you for supporting me and' }
    status { 'active' }
    gender { 'female' }

    after :build do |kid, evaluator|
      kid.name_pieces = build(:name_pieces) if unpopulated?(evaluator, :name_pieces)
      kid.address = build(:address) if unpopulated?(evaluator, :address)
      kid.diagnosis = build(:diagnosis, date: Faker::Date.between(from: kid.birthday, to: DateTime.now).to_s) if unpopulated?(evaluator, :diagnosis)
      kid.treatment_status = build(:treatment_status) if unpopulated?(evaluator, :treatment_status)
      kid.web_page = build(:web_page) if unpopulated?(evaluator, :web_page)
      kid.photos = build(:photos) if unpopulated?(evaluator, :photos)
      kid.relationships = [build(:kid_submitter)] if unpopulated?(evaluator, :relationships)
      kid.guardian = build(:guardian) if unpopulated?(evaluator, :guardian)
      kid.custom_institutions = [] if unpopulated?(evaluator, :custom_institutions)
    end
  end

  factory :kid_relationship, class: SBF::Client::Kid::Relationship do
    skip_create

    initialize_with do
      new(attributes)
    end

    type { SBF::Client::Kid::Relationship::Type::MOTHER }
    guardian_type { SBF::Client::Kid::Relationship::GuardianType::OTHER }
  end

  factory :kid_submitter, parent: :kid_relationship do
    is_submitter { true }

    after :build do |relationship, evaluator|
      relationship.permissions = {is_editor: true} if unpopulated?(evaluator, :permissions)
    end
  end

  factory :guardian, class: SBF::Client::Guardian do
    relationship { SBF::Client::Guardian::RelationshipType::MOTHER }

    after :build do |kid, evaluator|
      kid.name_pieces = build(:name_pieces) if unpopulated?(evaluator, :name_pieces)
      kid.email_address = build(:email_address) if unpopulated?(evaluator, :email_address)
    end
  end

  factory :kid_url, class: SBF::Client::KidURL do
    skip_create
    initialize_with do
      new(attributes)
    end

    url { Faker::Internet.url }
  end
end
