require_relative 'lib/helpers'

FactoryBot.define do
  factory :participant, class: SBF::Client::FullParticipant do
    to_create { |it| raise "Failed to create #{it.class}: #{it.errors.details}" unless it.save }

    initialize_with do
      new(attributes)
    end

    fundraising_goal { 100 }
    shave_time { Time.now.to_s }
    model_type { 'person' }
    fundraising_message { 'I am raising money for SBF' }
    how_created { 'admin_control_panel' }

    after :build do |participant, evaluator|
      participant.person = build(:person) if unpopulated?(evaluator, :person)
      if !unpopulated?(evaluator, :event) && evaluator.event.virtual?
        participant.is_virtual = true
        participant.shave_schedule = {date: Date.today + 30, time: '12:00:00'} if unpopulated?(evaluator, :shave_schedule)
      end
    end
  end

  factory :lead_organizer, class: SBF::Client::FullParticipant, parent: :participant do
    after :build do |participant, evaluator|
      participant.roles = build(:participant_roles, lead_organizer: true, shavee: false) if unpopulated?(evaluator, :roles)
    end
  end

  factory :organizer, class: SBF::Client::FullParticipant, parent: :participant do
    after :build do |participant, evaluator|
      participant.roles = build(:participant_roles, organizer: true, shavee: false) if unpopulated?(evaluator, :roles)
    end
  end

  factory :shavee, class: SBF::Client::FullParticipant, parent: :participant do
    after :build do |participant, evaluator|
      participant.roles = build(:participant_roles, shavee: true) if unpopulated?(evaluator, :roles)
    end
  end

  factory :team_captain, class: SBF::Client::FullParticipant, parent: :participant do
    after :build do |participant, evaluator|
      # NOTE: The roles are being ignored on purpose. Do not change this until team create through the api works - and when
      # that happens we will need to re-write the team factory. Instead we are using the move_participant_to_team route
      # which will create the roles for us.
      # participant.roles = build(:participant_roles, team_captain: true, shavee: true) if unpopulated?(evaluator, :roles)
      participant.roles = build(:participant_roles, team_captain: false, shavee: true) if unpopulated?(evaluator, :roles)
    end
  end

  factory :shavee_recruiter, class: SBF::Client::FullParticipant, parent: :participant do
    after :build do |participant, evaluator|
      participant.roles = build(:participant_roles, shavee_recruiter: true, shavee: false) if unpopulated?(evaluator, :roles)
    end
  end

  factory :treasurer, class: SBF::Client::FullParticipant, parent: :participant do
    after :build do |participant, evaluator|
      participant.roles = build(:participant_roles, treasurer: true, shavee: false) if unpopulated?(evaluator, :roles)
    end
  end

  factory :lead_treasurer, class: SBF::Client::FullParticipant, parent: :participant do
    after :build do |participant, evaluator|
      participant.roles = build(:participant_roles, lead_treasurer: true, shavee: false) if unpopulated?(evaluator, :roles)
    end
  end

  factory :barber_coordinator, class: SBF::Client::FullParticipant, parent: :participant do
    after :build do |participant, evaluator|
      participant.roles = build(:participant_roles, barber_coordinator: true, shavee: false) if unpopulated?(evaluator, :roles)
    end
  end

  factory :barber, class: SBF::Client::FullParticipant, parent: :participant do
    after :build do |participant, evaluator|
      participant.roles = build(:participant_roles, barber: true, shavee: false) if unpopulated?(evaluator, :roles)
    end
  end

  factory :event_support_volunteer, class: SBF::Client::FullParticipant, parent: :participant do
    after :build do |participant, evaluator|
      participant.roles = build(:participant_roles, event_support_volunteer: true, shavee: false) if unpopulated?(evaluator, :roles)
    end
  end

  factory :individual_fundraiser, class: SBF::Client::FullParticipant, parent: :participant do
    after :build do |participant, evaluator|
      participant.roles = build(:participant_roles, individual_fundraiser: true, shavee: false) if unpopulated?(evaluator, :roles)
    end
  end
end
