require_relative 'lib/helpers'

FactoryBot.define do
  factory :team, class: SBF::Client::FullTeam do
    to_create do |it|
      # Really gross - we don't support team create through the api currently.
      # Instead we need to use the start_team/move_participant_to_team logic

      # Save the participant first so we can move it
      raise "Failed to create #{it.class}: #{it.errors.details}" unless it.captain.save

      # Start the team using the info from the existing object and the participant we created
      response = SBF::Client::Participant.start_team(it.captain.id, it.name, it.fundraising_goal)
      raise "Failed to create #{it.class}: #{it.errors.details}" unless response.success?

      p = response.data
      raise "Failed to create #{it.class}: #{it.errors.details}" unless p.is_a?(SBF::Client::Participant)

      # Query the team on the participant that was returned (we are creating a team after all)
      team_hash = SBF::Client::Team.get(p.team.id).to_hash

      # Gross... find a better way eventually
      readonly_attributes = %w(display_name photos totals created_at modified_at)
      readonly_attributes.each { |attr| it.instance_variable_set("@#{attr}".to_sym, team_hash.delete(attr.to_sym)) }

      # Return the new team
      it.attributes = team_hash.merge(captain: it.captain)
    end

    initialize_with do
      new(attributes)
    end

    name { "#{Faker::Team.name} #{random}" }
    status { SBF::Client::Team::Status::ACTIVE }
    fundraising_goal { rand(50) * 500 }
    fundraising_message { 'We are raising money for SBF' }

    after :build do |team, evaluator|
      team.captain = build(:team_captain, event: team.event) if unpopulated?(evaluator, :captain)
    end
  end
end
